@@ -94,7 +94,7 @@ class LogStatus:
   def info(self, msg='', *args):
     '''
     To be Overloaded for MMI purpose.
-    Called by status() method and logCond()
+    Called by status(), worstStatus and logCond() methods
     '''
     self.msg = self.__class__.__name__  + ':' + msg.format(*args)
     if self.uselog:
@@ -112,7 +112,6 @@ class LogStatus:
     if msg:
       self.info(msg, *args)
     return self.ret
-    
 
   def logCond(self, cond, msg, *args):
     if cond:
