@@ -594,14 +594,20 @@ class Figure():
     return next( (m for m in self._lstMat if m.getName()==materialName), None)    
 
 
-  def addMaterial(self, mat:'PoserMaterial'):
+  def addMaterial(self, mat:'PoserMaterial', replace=False):
     # Check if it does not already exists and find the last material position
-    try:
-      next( m for m in self._lstMat if m.getName()==mat.getName())        
-      return C_FAIL
+    if not mat:
+      return C_ERROR
     
-    except StopIteration:   
-      self._lstMat.append(mat)
+    previous = next( (m for m in self._lstMat if m.getName()==mat.getName()), None)
+    if previous:
+      if replace:
+        self._lstMat.remove(previous)
+      else:
+        return C_FAIL
+      
+    # The material was not in the Figure list or has been removed, Add it
+    self._lstMat.append(mat)            
     
     return C_OK
   
@@ -1203,12 +1209,12 @@ class Figure():
         gt.setHidden( (hideComputed and ccd.lstOps) or ccd.isHidden )
         
         # Create the list of Alternate Geometries
-        if ccd.chanType == PoserToken.E_geomChan:
+        if ccd.chanType==PoserToken.E_geomChan:
           for fic in ccd.lstAltFiles:
             ccd.act.addAltGeom(altGeomFile=fic, poserRootDir=poserRootDir)
           ccd.max = len(ccd.act.getAltGeomList())
          
-        elif ccd.chanType== PoserToken.E_targetGeom: # Perform a Morph import
+        elif ccd.chanType==PoserToken.E_targetGeom: # Perform a Morph import
           wg = readGeom(ccd.lstAltFiles[0].getCanonicalPath())
           grp = wg.getGroup(ccd.groupName) if ccd.groupName else wg.getGroups()[0]
           if not grp:
@@ -1216,7 +1222,10 @@ class Figure():
             ret = min(ret, C_FAIL)
             continue
           ret = min(ret, ccd.act.createMorph(poserRootDir, grp, ccd.chanName, altGeomNo=ccd.altGeomNo)) # , minVectLen=minDist)
-          
+
+        elif ccd.chanType==PoserToken.E_shaderNodeParm:
+          # Link the Material.Node.NodeInput.parmR to the new channel
+          ccd.ni.setParmR(ccd.chanName)
 
         # The number of alternate geom, could have changed
         gt.setMax(ccd.max)
@@ -1298,13 +1307,6 @@ class Figure():
         driverChan = body.getGenericTransform(driverVPName)
         if not driverChan:
           driverChan = body.updateOrCreateVP(driverVPName, minVal=0.0, maxVal=1.0, applyLimits=True, isHidden=False)
-#           driverChan = GenericTransform(PoserToken.E_valueParm, driverVPName)
-#           driverChan.setPrintName(driverVPName)
-#           driverChan.setInitValue(1.0)
-#           driverChan.setMin(0.0)
-#           driverChan.setMax(1.0)
-#           driverChan.setTrackingScale(1.0)
-
           keys = Keys() # Default keys are static=0
           keys.addKey(0, 1.0)
           driverChan.addAttribut(keys)
@@ -1539,9 +1541,33 @@ class Figure():
         
       
     return nberrors
+  
  
- 
- 
+  def __copyMat(self, fp, lstMat):
+    if fp.name[0]=='{' and fp.name[-1]=='}':
+      try:                
+        mapMat = eval(fp.name)
+        if isinstance(mapMat, dict):                  
+          for srcMatName, destMatName in mapMat.items():                    
+            srcMat = fp.geomGroup.getMaterial(srcMatName) # , inCustom=True)
+            if srcMat:
+              # Rename the new material
+              nMat = PoserMaterial(src=srcMat)
+              nMat.setName(destMatName)
+              self.addMaterial(nMat, replace=True)
+            else:
+              fp.status(C_FAIL, 'Material Source [{:s}] not found in {:s}', srcMatName, fp.geomGroup.getName())
+        else:
+          fp.status(C_FAIL, 'Material Map [{:s}] Syntax Error:{:s}', fp.geomGroup.getName(), fp.name)
+          
+      except (SyntaxError, NameError) as se:
+        estr = 'Syntax Error in Material Map' + str(type(se)) + ' ==> ' + str(se.args) + '(' + str(se) + ')'      
+        fp.status(C_FAIL, 'Material Map [{:s}] {:s}', fp.geomGroup.getName(), estr)
+    else:
+      for mat in (lstMat if fp.name=='*' else [ fp.geomGroup.getMaterial(fp.name), ]):
+        self.addMaterial(PoserMaterial(src=mat), replace=True)
+
+    
  
   def createMechanical(self, lstParts:'list of FigurePart', optimize=True):
     ''' Create the actors described by a set of FigurePart(s) into the current figure.
@@ -1698,21 +1724,21 @@ class Figure():
       elif fp.partType[:3]=='mat': # Add a Material to the figure
         if isinstance(fp.geom, dict):
           for wfmat in fp.geom.values():
-            self.addMaterial(PoserMaterial(wfmat))
+            self.addMaterial(PoserMaterial(wfmat), replace=True)
           
         elif isinstance(fp.geomGroup, Figure):
-          lstMat = fp.geomGroup.getLstMaterial() if fp.name=='*' else [ fp.geomGroup.getMaterial(fp.name), ]
-          
-          for mat in lstMat: # Add each material in self
-            self.addMaterial(mat)
+          self.__copyMat(fp, fp.geomGroup.getLstMaterial())  
             
-        else: # partGeom is supposed to be an Actor/Prop
+        elif isinstance(fp.geomGroup, PoserMeshedObject): # partGeom is supposed to be an Actor/Prop
           if fp.geomGroup.getCustomMaterial():
-            for mat in [ a for a in fp.geomGroup.getLstAttr() \
-                         if (a.getPoserType()==PoserToken.E_material) and (fp.name=='*' or fp.name==a.getName()) ]:
-              self.addMaterial(mat)
+            self.__copyMat(fp, [ a for a in fp.geomGroup.getLstAttr() \
+                          if a.getPoserType()==PoserToken.E_material ] )  
           else:
             fp.status(C_FAIL, 'Actor [{:s}] has no custom material', fp.geomGroup.getName())
+            
+        else:
+          fp.status(C_FAIL, 'Actor [{:s}] illegal type', fp.geomGroup.getName())
+          
       else:
         fp.status(C_FAIL, 'Unknow part type:{:s}', fp.partType)
     #End of for 'FigurePart'
@@ -1728,6 +1754,7 @@ class Figure():
     return ret
     #End of createMechanical
  
+
   # Read Figure Description Chapter
   def read(self, st):
     code,cn,val = st.getLine()
@@ -1843,18 +1870,17 @@ class Figure():
       pa.writeData(fw, "")
 
     # Write FigureDescription part
-    fw.write(pfx + 'figure\n')
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
-    fw.write(nextPfx + "name " + self._printName + '\n')
-    fw.write(nextPfx + "root " + self.getRoot() + '\n')
+    fw.write(f"{pfx}figure\n{nextPfx}{{\n")
+    fw.write(f"{nextPfx}name {self._printName}\n")
+    fw.write(f"{nextPfx}root {self.getRoot()}\n")
     
     self._childs.writeData(fw, nextPfx)
 
     for po in self._inkyChains:        
       po.write(fw, nextPfx)
 
-    fw.write(nextPfx + "defaultPick " + self._defaultPick + '\n')
+    fw.write(f"{nextPfx}defaultPick {self._defaultPick}\n")
     WBoolLine(fw, nextPfx, "displayOn", self._displayOn)
 
     for po in self._linkParms:
@@ -1862,27 +1888,27 @@ class Figure():
 
     self._welds.writeData(fw, nextPfx)
 
-    fw.write(nextPfx + "allowsBending " + str(self._allowsBending) + '\n')
-    fw.write(nextPfx + "figureType " + str(self._figureType) + '\n')
-    fw.write(nextPfx + "origFigureType " + str(self._origFigureType) + '\n')
-    fw.write(nextPfx + "canonType " + str(self._canonType) + '\n')
+    fw.write(f"{nextPfx}allowsBending {self._allowsBending}\n")
+    fw.write(f"{nextPfx}figureType {self._figureType}\n")
+    fw.write(f"{nextPfx}origFigureType {self._origFigureType}\n")
+    fw.write(f"{nextPfx}canonType {self._canonType}\n")
     WBoolLine(fw, nextPfx, "conforming", self._conforming)
 
     for po in self._lstMat:
       po.write(fw, nextPfx)
 
     if self._displayMode:
-      fw.write(nextPfx + "displayMode " + self._displayMode + '\n')
+      fw.write(f"{nextPfx}displayMode {self._displayMode}\n")
 
     WBoolLine(fw, nextPfx, "locked", self._locked )
-    fw.write(nextPfx + "skinType " + str(self._skinType) + '\n')
+    fw.write(f"{nextPfx}skinType {self._skinType}\n")
     if self._attribute:
-      fw.write(nextPfx + "attribute " + self._attribute + '\n')
+      fw.write(f"{nextPfx}attribute {self._attribute}\n")
       
     # Write Custom Data if any  
     if self._customData:
       self._customData.write(fw, nextPfx)
       
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}}}\n")
 
 
