@@ -2,11 +2,10 @@
 # package: pftk.poser.kern
 from collections import namedtuple
 import logging
-import math
+import traceback, sys
 
 from langutil import RemoveExt, C_FAIL, C_OK, C_ERROR, C_FILE_NOT_FOUND
-from pypos3d.wftk.WFBasic import C_BAD_DELTAINDEX, C_BAD_DELTACOUNT, C_NODELTA, Vector3d, Point3d,\
-  FEPSILON
+from pypos3d.wftk.WFBasic import C_BAD_DELTAINDEX, C_BAD_DELTACOUNT, C_NODELTA, Vector3d, Point3d
 from pypos3d.wftk.PoserFileParser import PoserFileParser, ParsingErrorException
 from pypos3d.pftk.PoserBasic import PoserConst, PoserToken, PToken, buildRelPath, getRealPath, index, TAS, WBoolLine, create, Lang
 from pypos3d.pftk.GeomCustom import GeomCustom
@@ -135,6 +134,22 @@ class PoserMeshedObject(StructuredAttribut):
 
   def getFigure(self): return self._fig
 
+
+  def getMaterial(self, matName:'str', inCustom=False):
+    ''' Return the named material (None if it does not exists).
+    Limit search to CustomMaterial if inCustom=True
+    '''
+    mat = None
+    
+    if self.getCustomMaterial():
+      mat = next( (m for m in self.getLstAttr() \
+                  if (m.getPoserType()==PoserToken.E_material) and (matName=='*' or matName==m.getName())), None)
+        
+    if not mat and not inCustom and self.getFigure():
+      mat = self.getFigure().getMaterial(matName)
+          
+    return mat
+
   # Return the list of alternate geom (never null in Python)
   def getAltGeomList(self): return self._lstAltGeom
 
@@ -358,24 +373,7 @@ class PoserMeshedObject(StructuredAttribut):
           
         elif tor.lower().startswith('face='): # Can be Face=Material name
           vxtab = fp.geomGroup.getFaceVertexBy(tor[5:])
-          n = Point3d.triangle_normal(vxtab[0], vxtab[1], vxtab[2])
-          if AxisOrder=='yzx':
-            rotz = math.asin(-n.x)
-            cz   = math.cos(rotz)
-            rotx = math.acos(n.y / cz) if math.fabs(cz)>FEPSILON else 0.0 #
-          elif AxisOrder=='xyz':
-            roty = math.asin(n.z)
-            cy   = math.cos(roty)
-            rotz = math.acos(n.x/cy) if math.fabs(cy)>FEPSILON else 0.0
-          else: # 'zxy'
-            rotx = math.asin(-n.y)
-            cx   = math.cos(rotx)
-            roty = math.acos(n.z/cx) if math.fabs(cx)>FEPSILON else 0.0
-          
-          # Convert Result un degree
-          rotx = math.degrees(rotx)
-          roty = math.degrees(roty)
-          rotz = math.degrees(rotz)
+          rotx, roty, rotz = Point3d.triangle_orientation(vxtab[0], vxtab[1], vxtab[2], AxisOrder=AxisOrder, inDegrees=True)
         else:
           return fp.status(C_FAIL, 'Orientation definition error for [{:s}]:{:s}', self.getName(), tor)
         
@@ -431,8 +429,11 @@ class PoserMeshedObject(StructuredAttribut):
         vp = GenericTransform.createFrom(gtdef)
         self.addAttribut(vp)
   
-    except ValueError:
-      ret = fp.status(C_FAIL, 'Definition error for [{:s}]', self.getName())
+    except ValueError as ve:
+      exc_type, exc_value, exc_tb = sys.exc_info()
+      traceback.print_exception(exc_type, exc_value, exc_tb)
+       
+      ret = fp.status(C_FAIL, 'Definition error for [{:s}]:', self.getName(), str(ve))
 
     return ret
 
@@ -506,7 +507,8 @@ class PoserMeshedObject(StructuredAttribut):
     poserRootDir : str
       Base location of Poser Installation.
     bodyIdx : int
-      Index of the figure in the poser file
+      Index of the figure in the poser file.
+      Useless for a global prop
     lstch : List of channels.
       ChannelMorphStatusList or set of channel names
     Returns
@@ -1230,34 +1232,32 @@ class PoserMeshedObject(StructuredAttribut):
 
   # Write the definition part of the meshed object.
   def writeDef(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n{nextPfx}{{\n")
     if self._geomType==PoserConst.GT_NONE:
       fw.write('\n')
     elif self._geomType==PoserConst.GT_GLOBAL_OBJFILE:
-      fw.write(nextPfx + "storageOffset 0 0 0" + '\n')
-      fw.write(nextPfx + "geomHandlerGeom 13 " + self._geomGroupName + '\n')
+      fw.write(f"{nextPfx}storageOffset 0 0 0\n")
+      fw.write(f"{nextPfx}geomHandlerGeom 13 {self._geomGroupName}\n")
     elif self._geomType==PoserConst.GT_LOCAL_OBJFILE:
-      fw.write(nextPfx + "storageOffset 0 0.3487 0" + '\n')
-      fw.write(nextPfx + "objFileGeom 0 0 " + self._geomFileName + '\n')
+      fw.write(f"{nextPfx}storageOffset 0 0.3487 0\n")
+      fw.write(f"{nextPfx}objFileGeom 0 0 {self._geomFileName}\n")
     elif self._geomType==PoserConst.GT_INTERNAL:
       self._geomCustom.write(fw, nextPfx)
     else:
       logging.warning("Unknown GeomType at write")
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}}}\n")
 
   # Write the Data part of the meshed object.
   def writeData(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + " " + self.getName()+'\n')
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
-    fw.write(nextPfx + "name " + self._pmName+'\n')
-    fw.write(nextPfx + ("on" if self._visible else "off")+'\n')
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n{nextPfx}{{\n")
+    fw.write(f"{nextPfx}name {self._pmName}\n")
+    fw.write(f"{nextPfx}{'on' if self._visible else 'off'}\n")
     WBoolLine(fw, nextPfx, "bend", self._bend)
     
     if self.getPoserFile().isVersion(PoserConst.POSER_V9):
-        WBoolLine(fw, nextPfx, "animatableOrigin", self._animatableOrigin)
+      WBoolLine(fw, nextPfx, "animatableOrigin", self._animatableOrigin)
 
     WBoolLine(fw, nextPfx, "dynamicsLock", self._dynamicsLock)
 
@@ -1271,55 +1271,53 @@ class PoserMeshedObject(StructuredAttribut):
     WBoolLine(fw, nextPfx, "useZBuffer", self._useZBuffer )
 
     if self._parent:
-      fw.write(nextPfx + "parent " + self._parent+'\n')
+      fw.write(f"{nextPfx}parent {self._parent}\n")
     if self._conformingTarget:
-      fw.write(nextPfx + "conformingTarget " + self._conformingTarget+'\n')
+      fw.write(f"{nextPfx}conformingTarget {self._conformingTarget}\n")
     if self._lstAltGeom:
       for ag in self._lstAltGeom:
         ag.write(fw, nextPfx)
 
-    fw.write(nextPfx + "creaseAngle " + str(self._creaseAngle)+'\n')
+    fw.write(f"{nextPfx}creaseAngle {self._creaseAngle}\n")
     
     # Print WeightMap before channels
     for po in self._lstWMap:
       po.write(fw, nextPfx)
     
     # Channels { } part
-    # self._channels.write(fw, nextPfx)
-    fw.write(nextPfx + 'channels\n')
     chanPfx = nextPfx + "\t"
-    fw.write(chanPfx + "{\n")
+    fw.write(f"{nextPfx}channels\n{chanPfx}{{\n")
     if self._groups:
       self._groups.write(fw, chanPfx)
     for po in self.getChannels():
       po.write(fw, chanPfx)
-    fw.write(chanPfx + "}\n")
+    fw.write(f"{chanPfx}}}\n")
 
     if self._endPoint:
-        fw.write(nextPfx + "endPoint " + self._endPoint.poserPrint()+'\n')
+        fw.write(f"{nextPfx}endPoint {self._endPoint.poserPrint()}\n")
     if self._origin:
-        fw.write(nextPfx + "origin " + self._origin.poserPrint()+'\n')
+        fw.write(f"{nextPfx}origin {self._origin.poserPrint()}\n")
     if self._orientation:
-        fw.write(nextPfx + "orientation " + self._orientation.poserPrint()+'\n')
+        fw.write(f"{nextPfx}orientation {self._orientation.poserPrint()}\n")
 
     WBoolLine(fw, nextPfx, "displayOrigin", self._displayOrigin )
 
     if self._displayMode:
-        fw.write(nextPfx + "displayMode " + self._displayMode+'\n')
-    fw.write(nextPfx + "customMaterial " + str(self._customMaterial)+'\n')
+        fw.write(f"{nextPfx}displayMode {self._displayMode}\n")
+    fw.write(f"{nextPfx}customMaterial {self._customMaterial}\n")
     WBoolLine(fw, nextPfx, "locked", self._locked )
     
     # Print other found attributes
     for po in self._lstAttr:
       po.write(fw, nextPfx)
-    if len(self._lstAttr) == 0:
+    if not self._lstAttr:
       fw.write('\n')
       
     # Write Customer Data
     if self._customData:
       self._customData.write(fw, nextPfx)
 
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}}}\n")
 
   def write(self, fw, pfx):
     raise Exception(PoserConst.C_EX_NOTCALL)
@@ -1426,8 +1424,8 @@ class BaseProp(PoserProp):
 
   #    * @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
-    if len(self._lstAttr) == 0:
-      fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
+    if not self._lstAttr:
+      fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
     else:
       super(BaseProp, self).write(fw, pfx)
 
@@ -1493,8 +1491,8 @@ class HairProp(PoserProp):
 
   #    * @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
-    if self._isSimple:
-      fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
+    if self._isSimple:    
+      fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
 
   def addAttribut(self, po):
     super(HairProp, self).addAttribut(po)
