@@ -17,8 +17,7 @@ from pypos3d.wftk.Repere import Repere
 from pypos3d.wftk.PoserFileParser import PoserFileParser, ParsingErrorException
 from pypos3d.wftk.WaveGeom import WFMat, WFColor
 from pypos3d.pftk.PoserBasic import PoserConst, PoserObject, PToken, PoserToken, Lang, TAS, index, create, WBoolLine, RemoveQuotes, nodeNameNo
-from pypos3d.pftk.SimpleAttribut import KSA, ValueOpDelta, OffSA, TextureMapSA,\
-  SimpleAttribut
+from pypos3d.pftk.SimpleAttribut import KSA, ValueOpDelta, OffSA, TextureMapSA, SimpleAttribut
 
  
 PtMapping = namedtuple('PtMapping', ['srcNo', 'refNo', 'refGeom', 'dist' ])
@@ -81,28 +80,24 @@ class StructuredAttribut(PoserObject):
 
   # @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n{nextPfx}{{\n")
     for po in self._lstAttr:
       po.write(fw, nextPfx)
 
-    if len(self._lstAttr) == 0:
+    if not self._lstAttr:
       fw.write('\n')
 
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}}}\n")
 
   def getAttribut(self, attrTypeOrName):
-    try:
-      if isinstance(attrTypeOrName, PToken):
-        po = next(x for x in self._lstAttr if x.getPoserType() == attrTypeOrName)
-      else:
-        po = next(x for x in self._lstAttr if x.getName() == attrTypeOrName)
+    if isinstance(attrTypeOrName, PToken):
+      po = next((x for x in self._lstAttr if x.getPoserType() == attrTypeOrName), None)
+    else:
+      po = next((x for x in self._lstAttr if x.getName() == attrTypeOrName), None)
 
-      return po
-    except StopIteration:
-      #  20080417 : Fix - Shall return null if not found
-      return None
+    return po
+    #  20080417 : Fix - Shall return null if not found
 
 # Useless as we always use less 'free' attributs
 #   def replaceAttribut(self, attrTypeName, nattr):
@@ -202,15 +197,14 @@ class VertsGroup(StructuredAttribut):
   #    * @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
     #  FIXME: Wrong output - Other attributes (like stitchVertsGroupProperties are forgotten)
-    fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n{nextPfx}{{\n")
     nextPfx = pfx + "\tv "
 
     for vi in self.vertTab:
-      fw.write(nextPfx + str(vi) + '\n')
+      fw.write(f"{nextPfx}{vi}\n")
  
-    fw.write(pfx + "\t}\n")
+    fw.write(f"{nextPfx}}}\n")
 
 # 
 # This class represents the "textureFile" element of Poser files.
@@ -234,16 +228,18 @@ class TextureFile(StructuredAttribut):
   #    * @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   #    
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
-    fw.write(nextPfx + "file " + (PoserConst.C_NO_MAPG if (self._file == None) else self._file) + '\n') #  "NO_MAP" 
+    fw.write(f"{nextPfx}{{\n")
+    fw.write(f"{nextPfx}file {PoserConst.C_NO_MAPG if self._file==None else self._file}\n") #  "NO_MAP" 
     WBoolLine(fw, nextPfx, "warped", self._warped )
     for po in self._lstAttr:
       po.write(fw, nextPfx)
-    if len(self._lstAttr) == 0:
+    if not self._lstAttr:
       fw.write('\n')
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}}}\n")
+
+ 
 
 # 
 # This class represents the "nodeInput" element of Poser files.
@@ -255,9 +251,7 @@ class NodeInput(StructuredAttribut):
     
     if src:
       self._nodeName = src._nodeName
-      if nodeName:
-        self._name = nodeName
-        
+      self._name = nodeName if nodeName else src._name
       self._value    = src._value
       self._parmR    = src._parmR
       self._parmG    = src._parmG
@@ -321,17 +315,20 @@ class NodeInput(StructuredAttribut):
   #    * @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   #    
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + ' "' + self.getName() +'"\n')
+    fw.write(f"{pfx}{self.getPoserType().token} \"{self.getName()}\"\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + '{\n')
-    fw.write(nextPfx + 'name "' + self._nodeName +'"\n')
-    fw.write(nextPfx + "value " + self._value +'\n')
-    fw.write(nextPfx + "parmR " + (self._parmR if self._parmR else PoserConst.C_NO_PARM ) +'\n')
-    fw.write(nextPfx + "parmG " + (self._parmG if self._parmG else PoserConst.C_NO_PARM ) +'\n')
-    fw.write(nextPfx + "parmB " + (self._parmB if self._parmB else PoserConst.C_NO_PARM ) +'\n')
-    fw.write(nextPfx + 'node "' + (self._node if self._node else PoserConst.C_NO_NODE )+'"\n')
-    fw.write(nextPfx + "file " + (self._file if self._file else PoserConst.C_NO_MAPG ) +'\n')
-    fw.write(nextPfx + '}\n')
+    fw.write(f"{nextPfx}{{\n")
+    fw.write(f"{nextPfx}name \"{self._nodeName}\"\n")
+    fw.write(f"{nextPfx}value {self._value}\n")
+    fw.write(f"{nextPfx}parmR {self._parmR if self._parmR else PoserConst.C_NO_PARM}\n")
+    fw.write(f"{nextPfx}parmG {self._parmG if self._parmG else PoserConst.C_NO_PARM}\n")
+    fw.write(f"{nextPfx}parmB {self._parmB if self._parmB else PoserConst.C_NO_PARM}\n")
+    fw.write(f"{nextPfx}node \"{self._node if self._node else PoserConst.C_NO_NODE}\"\n")
+    fw.write(f"{nextPfx}file {self._file if self._file else PoserConst.C_NO_MAPG}\n")
+    fw.write(f"{nextPfx}}}\n")
+
+
+
 
   def getFile(self): return self._file
 
@@ -350,6 +347,9 @@ class NodeInput(StructuredAttribut):
     t = self._value.split(' ')
     self._value = str(f) + ' ' + t[1] + ' ' + t[2]
 
+  def setParmR(self, s):
+    self._parmR = s
+
   def getInNode(self, sht):
     if self._node and self._node!=PoserConst.C_NO_NODE:
       return sht.getNodeByInternalName(self._node)
@@ -363,7 +363,10 @@ class Node(StructuredAttribut):
     super(Node, self).__init__()
     self._poserType = PoserToken.E_node
     self.ambiguious = True
+    
+    # Copy constructor
     if src:
+      self._name      = src._name
       self._nodeName  = src._nodeName
       self._typeName  = src._typeName
       self._pos       = src._pos
@@ -415,12 +418,12 @@ class Node(StructuredAttribut):
 
   # @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + ' "' + self._typeName + '" "' + self.getName() + '"\n')
+    fw.write(f"{pfx}{self.getPoserType().token} \"{self._typeName}\" \"{self.getName()}\"\n")
     if self._lstAttr:
       nextPfx = pfx + "\t"
-      fw.write(nextPfx + "{\n")
-      fw.write(nextPfx + 'name "' + self._nodeName + '"\n')
-      fw.write(nextPfx + "pos " + self._pos + '\n')
+      fw.write(f"{nextPfx}{{\n")
+      fw.write(f"{nextPfx}name \"{self._nodeName}\"\n")
+      fw.write(f"{nextPfx}pos {self._pos}\n")
 
       if self._collapsed:
         WBoolLine(fw, nextPfx, "inputsCollapsed", self._collapsed)
@@ -429,15 +432,15 @@ class Node(StructuredAttribut):
         
       for po in self._lstAttr:
         po.write(fw, nextPfx)
-#       if len(self._lstAttr)==0:
-#         fw.write('\n')
-      fw.write(nextPfx + "}\n")
+        
+      fw.write(f"{nextPfx}}}\n")
 
   def getInputByInternalName(self, inpName='Blending'):
-    for no in self.getLstAttr():
-      if isinstance(no, NodeInput) and no._nodeName==inpName:
-        return no
-    return None
+    return next( (n for n in self.getLstAttr() if isinstance(n, NodeInput) and n._nodeName==inpName), None)
+
+#   def getInputByName(self, name):
+#     return next( (n for n in self.getLstAttr() if isinstance(n, NodeInput) and n.getName()==name), None)
+
 
   def setLocation(self, x, y):
     self._pos = str(x)+ ' ' + str(y)
@@ -451,12 +454,14 @@ class Node(StructuredAttribut):
 class ShaderTree(StructuredAttribut):
   def __init__(self):
     super(ShaderTree, self).__init__()
+    self.setPoserType(PoserToken.E_shaderTree)
 
   def getNodeByInternalName(self, intName):
-    for po in self.getLstAttr():
-      if isinstance(po, Node) and po._nodeName==intName:
-        return po
-    return None
+    return next( (n for n in self.getLstAttr() if isinstance(n, Node) and n._nodeName==intName), None)
+
+  def getNodeByName(self, name):
+    return next( (n for n in self.getLstAttr() if isinstance(n, Node) and n.getName()==name), None)
+
 
   # Create a Node in a shader tree
   # Partial Implementation for 'blender' and 'image_map'
@@ -700,30 +705,30 @@ class PoserMaterial(StructuredAttribut):
 
   # @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + ' ' + self.getName() + '\n')
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
-    fw.write(nextPfx + 'KdColor {:f} {:f} {:f} {:f}\n'.format(self._KdColor.r, self._KdColor.g, self._KdColor.b, self._KdColor.t)) 
-    fw.write(nextPfx + 'KaColor {:f} {:f} {:f} {:f}\n'.format(self._KaColor.r, self._KaColor.g, self._KaColor.b, self._KaColor.t)) 
-    fw.write(nextPfx + 'KsColor {:f} {:f} {:f} {:f}\n'.format(self._KsColor.r, self._KsColor.g, self._KsColor.b, self._KsColor.t)) 
-    fw.write(nextPfx + 'TextureColor {:g} {:g} {:g} {:g}\n'.format(self._TextureColor.r, self._TextureColor.g, self._TextureColor.b, self._TextureColor.t)) 
+    fw.write(f"{nextPfx}{{\n")
+    fw.write(f"{nextPfx}KdColor {self._KdColor.r} {self._KdColor.g} {self._KdColor.b} {self._KdColor.t}\n") 
+    fw.write(f"{nextPfx}KaColor {self._KaColor.r} {self._KaColor.g} {self._KaColor.b} {self._KaColor.t}\n")
+    fw.write(f"{nextPfx}KsColor {self._KsColor.r} {self._KsColor.g} {self._KsColor.b} {self._KsColor.t}\n")
+    fw.write(f"{nextPfx}TextureColor {self._TextureColor.r} {self._TextureColor.g} {self._TextureColor.b} {self._TextureColor.t}\n") 
 
-    fw.write(nextPfx + 'NsExponent {:f}\n'.format(self._NsExponent)) 
-    fw.write(nextPfx + 'tMin {:f}\n'.format(self._tMin)) 
-    fw.write(nextPfx + 'tMax {:f}\n'.format(self._tMax)) 
-    fw.write(nextPfx + 'tExpo {:f}\n'.format(self._tExpo)) 
-    fw.write(nextPfx + 'bumpStrength {:f}\n'.format(self._bumpStrength)) 
-    fw.write(nextPfx + 'ksIgnoreTexture {:d}\n'.format(self._ksIgnoreTexture)) 
-    fw.write(nextPfx + 'reflectThruLights {:d}\n'.format(self._reflectThruLights)) 
-    fw.write(nextPfx + 'reflectThruKd {:f}\n'.format(self._reflectThruKd)) 
+    fw.write(f"{nextPfx}NsExponent {self._NsExponent}\n") 
+    fw.write(f"{nextPfx}tMin {self._tMin}\n") 
+    fw.write(f"{nextPfx}tMax {self._tMax}\n") 
+    fw.write(f"{nextPfx}tExpo {self._tExpo}\n") 
+    fw.write(f"{nextPfx}bumpStrength {self._bumpStrength}\n") 
+    fw.write(f"{nextPfx}ksIgnoreTexture {self._ksIgnoreTexture}\n") 
+    fw.write(f"{nextPfx}reflectThruLights {self._reflectThruLights}\n") 
+    fw.write(f"{nextPfx}reflectThruKd {self._reflectThruKd}\n") 
     
     self._textureMap.write(fw, nextPfx)
     self._bumpMap.write(fw, nextPfx)
     self._reflectionMap.write(fw, nextPfx)
     self._transparencyMap.write(fw, nextPfx)
 
-    fw.write(nextPfx + 'ReflectionColor {:f} {:f} {:f} {:f}\n'.format(self._ReflectionColor.r, self._ReflectionColor.g, self._ReflectionColor.b, self._ReflectionColor.t)) 
-    fw.write(nextPfx + 'reflectionStrength {:f}\n'.format(self._reflectionStrength)) 
+    fw.write(f"{nextPfx}ReflectionColor {self._ReflectionColor.r} {self._ReflectionColor.g} {self._ReflectionColor.b} {self._ReflectionColor.t}\n") 
+    fw.write(f"{nextPfx}reflectionStrength {self._reflectionStrength}\n") 
 
     if self._shaderTree:
       self._shaderTree.write(fw, nextPfx)
@@ -732,8 +737,7 @@ class PoserMaterial(StructuredAttribut):
       for po in self._lstAttr:
         po.write(fw, nextPfx)
 
-    fw.write(nextPfx + "}\n")
-
+    fw.write(f"{nextPfx}}}\n")
 
 
 # 
@@ -767,16 +771,16 @@ class InkyChain(StructuredAttribut):
   #    * @see pftk.poser.kern.StructuredAttribut#write(java.io.PrintWriter,
   #    
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + " " + self.getName()+'\n')
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
-    fw.write(nextPfx + "name " + self._pmName + '\n')
-    fw.write(nextPfx + ("on\n" if self._active else "off\n"))
+    fw.write(f"{nextPfx}{{\n")
+    fw.write(f"{nextPfx}name {self._pmName}\n")
+    fw.write(f"{nextPfx}{'on' if self._active else 'off'}\n")
     for po in self._lstAttr:
       po.write(fw, nextPfx)
-    if len(self._lstAttr) == 0:
+    if not self._lstAttr:
       fw.write('\n')
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}}}\n")
 
 # 
 #  * This class represents the "keys" entry of every channel definition.
@@ -849,13 +853,13 @@ class Keys(StructuredAttribut):
 
 
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token+'\n')
+    fw.write(f"{pfx}{self.getPoserType().token}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
+    fw.write(f"{nextPfx}{{\n")
     WBoolLine(fw, nextPfx, "static", self._static )
     for ks in sorted(self._hshKeys.values(), key=lambda k:k._noFrame):
       ks.write(fw, nextPfx)
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}}}\n")
 
   def setStatic(self, s):
     self._static = s
@@ -934,22 +938,22 @@ class DocDescription(StructuredAttribut):
 
   #    * @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token+ '\n')
+    fw.write(f"{pfx}{self.getPoserType().token}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
+    fw.write(f"{nextPfx}{{\n")
     if self._dimensions:
-      fw.write(nextPfx + "dimensions " + self._dimensions + '\n')
+      fw.write(f"{nextPfx}dimensions {self._dimensions}\n")
 
     if self._screenPlace:
-      fw.write(nextPfx + "screenPlace " + self._screenPlace + '\n')
+      fw.write(f"{nextPfx}screenPlace {self._screenPlace}\n")
 
     if self._displayMode:
-      fw.write(nextPfx + "displayMode " + self._displayMode + '\n')
+      fw.write(f"{nextPfx}displayMode {self._displayMode}\n")
 
     for po in self._lstAttr:
       po.write(fw, nextPfx)
 
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}}}\n")
 
 # Description part of a figure.
 #  class FigureDescription(StructuredAttribut): --> inlined in Figure
@@ -976,12 +980,13 @@ class AlternateGeom(StructuredAttribut):
 
   #  @see pftk.poser.kern.StructuredAttribut#write(java.io.PrintWriter,
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
-    fw.write(nextPfx + "name " + self._printName +'\n')
-    fw.write(nextPfx + "objFile 20 " + self._geomFileName +'\n')
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}{{\n")
+    fw.write(f"{nextPfx}name {self._printName}\n")
+    fw.write(f"{nextPfx}objFile 20 {self._geomFileName}\n")
+    fw.write(f"{nextPfx}}}\n")
+
 
   def setNo(self, n):
     self._no = n
@@ -1059,12 +1064,12 @@ class Deltas(StructuredAttribut):
 
   # @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + '\n')
+    fw.write(f"{pfx}{self.getPoserType().token}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
+    fw.write(f"{nextPfx}{{\n")
     for d in sorted(self.deltaSet.values(), key=lambda p:p.noPt):
-      fw.write(nextPfx + str(d) + '\n')
-    fw.write(nextPfx + "}\n")
+      fw.write(f"{nextPfx}{d}\n")
+    fw.write(f"{nextPfx}}}\n")
 
   def clear(self):
     self.deltaSet.clear() # = { }
@@ -1460,37 +1465,38 @@ class GenericTransform(StructuredAttribut):
 
   #    * @see pftk.poser.kern.StructuredAttribut#write(java.io.PrintWriter,
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + " " + self.getName() +'\n')
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + '{\n')
-    fw.write(nextPfx + "name " + self._transfName +'\n')
-    fw.write(nextPfx + "initValue " + str(self._initValue) +'\n')
-    #fw.write(nextPfx + "hidden " + (1 if self._hidden else 0) +'\n')
+    fw.write(f"{nextPfx}{{\n")
+    fw.write(f"{nextPfx}name {self._transfName}\n")
+    fw.write(f"{nextPfx}initValue {self._initValue}\n")
     WBoolLine(fw, nextPfx, "hidden", self._hidden )
     
     #  Test Poser language version before writing
     if self._pmo and (self._pmo.getPoserFile().isVersion(PoserConst.POSER_V9)):
       WBoolLine(fw, nextPfx, "enabled", self._enabled )
-    fw.write(nextPfx + "forceLimits " + str(self._forceLimits) +'\n')
-    fw.write(nextPfx + "min " + str(self._min) +'\n')
-    fw.write(nextPfx + "max " + str(self._max) +'\n')
-    fw.write(nextPfx + "trackingScale " + str(self._trackingScale) +'\n')
+    fw.write(f"{nextPfx}forceLimits {self._forceLimits}\n")
+    fw.write(f"{nextPfx}min {self._min}\n")
+    fw.write(f"{nextPfx}max {self._max}\n")
+    fw.write(f"{nextPfx}trackingScale {self._trackingScale}\n")
     #  Print Keys
     if self._keys != None:
       self._keys.write(fw, nextPfx)
     if self._interpStyleLocked != -sys.maxsize:
-      fw.write(nextPfx + "interpStyleLocked " + str(self._interpStyleLocked) +'\n')
+      fw.write(f"{nextPfx}interpStyleLocked {self._interpStyleLocked}\n")
     if self._hasStaticValue:
-      fw.write(nextPfx + "staticValue " + str(self._staticValue) +'\n')
+      fw.write(f"{nextPfx}staticValue {self._staticValue}\n")
     for vod in self._lstVOD:
       vod.write(fw, nextPfx)
     for po in self._lstAttr:
       po.write(fw, nextPfx)
     if self._dlt and (len(self._dlt.deltaSet)>0):
-      fw.write(nextPfx + "indexes " + str(len(self._dlt.deltaSet)) + '\n')
-      fw.write(nextPfx + "numbDeltas " + str(self._numbDeltas) +'\n')
+      fw.write(f"{nextPfx}indexes {len(self._dlt.deltaSet)}\n")
+      fw.write(f"{nextPfx}numbDeltas {self._numbDeltas}\n")
       self._dlt.write(fw, nextPfx)
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}}}\n")
+
+
 
   def setPoserMeshedObject(self, pmo):
     self._pmo = pmo
@@ -1611,7 +1617,7 @@ class DepthMapSize(GenericTransform):
   #    * @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
     if self._isSimple:
-      fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
+      fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
     else:
       super(DepthMapSize, self).write(fw, pfx)
 
@@ -1815,17 +1821,17 @@ class FrontImage(StructuredAttribut):
 
   #    * @see deyme.v3d.poser.PoserObject#write(java.io.FileWriter)
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + " " + self.getName()+'\n')
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
-    fw.write(nextPfx + "loaded " + ("1" if self._loaded else "0") + '\n')
-    fw.write(nextPfx + "file " + (self._file if self._file else PoserConst.C_NO_MAPG ) + '\n')
-    fw.write(nextPfx + "flipped " + ("1" if self._flipped else "0")+ '\n')
+    fw.write(f"{nextPfx}{{\n")
+    fw.write(f"{nextPfx}loaded {1 if self._loaded else 0}\n")
+    fw.write(f"{nextPfx}file {self._file if self._file else PoserConst.C_NO_MAPG}\n")
+    fw.write(f"{nextPfx}flipped {1 if self._flipped else 0}\n")
     for po in self._lstAttr:
       po.write(fw, nextPfx)
-    if len(self._lstAttr) == 0:
+    if not self._lstAttr:
       fw.write('\n')
-    fw.write(nextPfx + "}\n")
+    fw.write(f"{nextPfx}}}\n")
 
 # package: pftk.poser.kern
 class SideImage(FrontImage):
@@ -1863,17 +1869,16 @@ class WeightMap(StructuredAttribut):
     # End of read
 
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
-    fw.write(nextPfx + "numbVerts " + str(self._numbVerts)+'\n')
-
-    nextPfx = pfx + "\tv {0:d} {1:9.6f}\n"
-
+    fw.write(f"{nextPfx}{{\n")
+    fw.write(f"{nextPfx}numbVerts {self._numbVerts}\n")
     for vi in self._vertTab:
-      fw.write(nextPfx.format(vi[0], vi[1]))
+      fw.write(f"{pfx}\tv {vi[0]:d} {vi[1]:9.6f}\n")
+
+    fw.write(f"{pfx}\t}}\n")
  
-    fw.write(pfx + "\t}\n")
+
 
 class Zones(StructuredAttribut):
   def __init__(self, Name=None):
@@ -1925,16 +1930,16 @@ class CustomData(StructuredAttribut):
     # End of read
 
   def write(self, fw, pfx):
-    fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
+    fw.write(f"{pfx}{self.getPoserType().token} {self.getName()}\n")
     nextPfx = pfx + "\t"
-    fw.write(nextPfx + "{\n")
+    fw.write(f"{nextPfx}{{\n")
 
     nextPfx = pfx + '\tdata {0:s} {1:d} {2:d} "{3:s}"\n'
 
     for k,d in self._data.items():
       fw.write(nextPfx.format(k, d.withPose, d.withMat, d.val))
  
-    fw.write(pfx + "\t}\n")
+    fw.write(f"{pfx}\t}}\n")
 
 
 # 
